@@ -14,7 +14,7 @@
  *  limitations under the License.
  *
  */
-#include "luv.h"
+#include "private.h"
 
 static uv_check_t* luv_check_check(lua_State* L, int index) {
   uv_check_t* handle = (uv_check_t*)luv_checkudata(L, index, "uv_check");
@@ -45,16 +45,12 @@ static int luv_check_start(lua_State* L) {
   int ret;
   luv_check_callback(L, (luv_handle_t *)handle->data, LUV_CHECK, 2);
   ret = uv_check_start(handle, luv_check_cb);
-  if (ret < 0) return luv_error(L, ret);
-  lua_pushinteger(L, ret);
-  return 1;
+  return luv_result(L, ret);
 }
 
 static int luv_check_stop(lua_State* L) {
   uv_check_t* handle = luv_check_check(L, 1);
   int ret = uv_check_stop(handle);
-  if (ret < 0) return luv_error(L, ret);
-  lua_pushinteger(L, ret);
-  return 1;
+  return luv_result(L, ret);
 }
 
