@@ -14,7 +14,7 @@
  *  limitations under the License.
  *
  */
-#include "luv.h"
+#include "private.h"
 
 static void* luv_newuserdata(lua_State* L, size_t sz) {
   void* handle = malloc(sz);
@@ -97,7 +97,8 @@ static int luv_close(lua_State* L) {
 static void luv_handle_free(uv_handle_t* handle) {
   luv_handle_t* data = (luv_handle_t*)handle->data;
   if (data) {
-    free(data->extra);
+    if (data->extra_gc)
+      data->extra_gc(data->extra);
     free(data);
   }
   free(handle);
@@ -151,34 +152,32 @@ static int luv_has_ref(lua_State* L) {
 
 static int luv_send_buffer_size(lua_State* L) {
   uv_handle_t* handle = luv_check_handle(L, 1);
-  int value;
+  int value = luaL_optinteger(L, 2, 0);
   int ret;
-  if (lua_isnoneornil(L, 2)) {
-    value = 0;
+  if (value == 0) { // get
+    ret = uv_send_buffer_size(handle, &value);
+    if (ret < 0) return luv_error(L, ret);
+    lua_pushinteger(L, value);
+    return 1;
+  } else { // set
+    ret = uv_send_buffer_size(handle, &value);
+    return luv_result(L, ret);
   }
-  else {
-    value = luaL_checkinteger(L, 2);
-  }
-  ret = uv_send_buffer_size(handle, &value);
-  if (ret < 0) return luv_error(L, ret);
-  lua_pushinteger(L, ret);
-  return 1;
 }
 
 static int luv_recv_buffer_size(lua_State* L) {
   uv_handle_t* handle = luv_check_handle(L, 1);
-  int value;
+  int value = luaL_optinteger(L, 2, 0);
   int ret;
-  if (lua_isnoneornil(L, 2)) {
-    value = 0;
+  if (value == 0) { // get
+    ret = uv_recv_buffer_size(handle, &value);
+    if (ret < 0) return luv_error(L, ret);
+    lua_pushinteger(L, value);
+    return 1;
+  } else { // set
+    ret = uv_recv_buffer_size(handle, &value);
+    return luv_result(L, ret);
   }
-  else {
-    value = luaL_checkinteger(L, 2);
-  }
-  ret = uv_recv_buffer_size(handle, &value);
-  if (ret < 0) return luv_error(L, ret);
-  lua_pushinteger(L, ret);
-  return 1;
 }
 
 static int luv_fileno(lua_State* L) {
