@@ -14,7 +14,7 @@
  *  limitations under the License.
  *
  */
-#include "lreq.h"
+#include "private.h"
 
 
 static int luv_check_continuation(lua_State* L, int index) {
@@ -64,9 +64,16 @@ static void luv_fulfill_req(lua_State* L, luv_req_t* data, int nargs) {
 }
 
 static void luv_cleanup_req(lua_State* L, luv_req_t* data) {
+  int i;
   luaL_unref(L, LUA_REGISTRYINDEX, data->req_ref);
   luaL_unref(L, LUA_REGISTRYINDEX, data->callback_ref);
-  luaL_unref(L, LUA_REGISTRYINDEX, data->data_ref);
+  if (data->data_ref == LUV_REQ_MULTIREF) {
+    for (i = 0; ((int*)(data->data))[i] != LUA_NOREF; i++) {
+      luaL_unref(L, LUA_REGISTRYINDEX, ((int*)(data->data))[i]);
+    }
+  }
+  else
+    luaL_unref(L, LUA_REGISTRYINDEX, data->data_ref);
   free(data->data);
   free(data);
 }
