@@ -17,9 +17,8 @@
 
 #include <lua.h>
 #if (LUA_VERSION_NUM != 503)
-#include "c-api/compat-5.3.h"
+#include "compat-5.3.h"
 #endif
-#define LUV_SOURCE
 #include "luv.h"
 
 #include "util.c"
@@ -135,6 +134,9 @@ static const luaL_Reg luv_functions[] = {
   {"is_readable", luv_is_readable},
   {"is_writable", luv_is_writable},
   {"stream_set_blocking", luv_stream_set_blocking},
+#if LUV_UV_VERSION_GEQ(1, 19, 0)
+  {"stream_get_write_queue_size", luv_stream_get_write_queue_size},
+#endif
 
   // tcp.c
   {"new_tcp", luv_new_tcp},
@@ -147,6 +149,9 @@ static const luaL_Reg luv_functions[] = {
   {"tcp_getsockname", luv_tcp_getsockname},
   {"tcp_connect", luv_tcp_connect},
   {"tcp_write_queue_size", luv_write_queue_size},
+#if LUV_UV_VERSION_GEQ(1, 32, 0)
+  {"tcp_close_reset", luv_tcp_close_reset},
+#endif
 
   // pipe.c
   {"new_pipe", luv_new_pipe},
@@ -154,7 +159,9 @@ static const luaL_Reg luv_functions[] = {
   {"pipe_bind", luv_pipe_bind},
   {"pipe_connect", luv_pipe_connect},
   {"pipe_getsockname", luv_pipe_getsockname},
+#if LUV_UV_VERSION_GEQ(1, 3, 0)
   {"pipe_getpeername", luv_pipe_getpeername},
+#endif
   {"pipe_pending_instances", luv_pipe_pending_instances},
   {"pipe_pending_count", luv_pipe_pending_count},
   {"pipe_pending_type", luv_pipe_pending_type},
@@ -164,6 +171,10 @@ static const luaL_Reg luv_functions[] = {
   {"tty_set_mode", luv_tty_set_mode},
   {"tty_reset_mode", luv_tty_reset_mode},
   {"tty_get_winsize", luv_tty_get_winsize},
+#if LUV_UV_VERSION_GEQ(1, 33, 0)
+  {"tty_set_vterm_state", luv_tty_set_vterm_state},
+  {"tty_get_vterm_state", luv_tty_get_vterm_state},
+#endif
 
   // udp.c
   {"new_udp", luv_new_udp},
@@ -207,6 +218,9 @@ static const luaL_Reg luv_functions[] = {
   {"fs_write", luv_fs_write},
   {"fs_mkdir", luv_fs_mkdir},
   {"fs_mkdtemp", luv_fs_mkdtemp},
+#if LUV_UV_VERSION_GEQ(1, 34, 0)
+  {"fs_mkstemp", luv_fs_mkstemp},
+#endif
   {"fs_rmdir", luv_fs_rmdir},
   {"fs_scandir", luv_fs_scandir},
   {"fs_scandir_next", luv_fs_scandir_next},
@@ -223,12 +237,20 @@ static const luaL_Reg luv_functions[] = {
   {"fs_fchmod", luv_fs_fchmod},
   {"fs_utime", luv_fs_utime},
   {"fs_futime", luv_fs_futime},
+#if LUV_UV_VERSION_GEQ(1, 36, 0)
+  {"fs_lutime", luv_fs_lutime},
+#endif
   {"fs_link", luv_fs_link},
   {"fs_symlink", luv_fs_symlink},
   {"fs_readlink", luv_fs_readlink},
+#if LUV_UV_VERSION_GEQ(1, 8, 0)
   {"fs_realpath", luv_fs_realpath},
+#endif
   {"fs_chown", luv_fs_chown},
   {"fs_fchown", luv_fs_fchown},
+#if LUV_UV_VERSION_GEQ(1, 21, 0)
+  {"fs_lchown", luv_fs_lchown},
+#endif
 #if LUV_UV_VERSION_GEQ(1, 14, 0)
   {"fs_copyfile", luv_fs_copyfile },
 #endif
@@ -237,6 +259,9 @@ static const luaL_Reg luv_functions[] = {
   {"fs_readdir", luv_fs_readdir},
   {"fs_closedir", luv_fs_closedir},
 #endif
+#if LUV_UV_VERSION_GEQ(1, 31, 0)
+  {"fs_statfs", luv_fs_statfs},
+#endif
 
   // dns.c
   {"getaddrinfo", luv_getaddrinfo},
@@ -244,9 +269,11 @@ static const luaL_Reg luv_functions[] = {
 
   // misc.c
   {"chdir", luv_chdir},
+#if LUV_UV_VERSION_GEQ(1, 9, 0)
   {"os_homedir", luv_os_homedir},
   {"os_tmpdir", luv_os_tmpdir},
   {"os_get_passwd", luv_os_get_passwd},
+#endif
   {"cpu_info", luv_cpu_info},
   {"cwd", luv_cwd},
   {"exepath", luv_exepath},
@@ -274,9 +301,11 @@ static const luaL_Reg luv_functions[] = {
   {"version", luv_version},
   {"version_string", luv_version_string},
 #ifndef _WIN32
+#if LUV_UV_VERSION_GEQ(1, 8, 0)
   {"print_all_handles", luv_print_all_handles},
   {"print_active_handles", luv_print_active_handles},
 #endif
+#endif
 #if LUV_UV_VERSION_GEQ(1, 12, 0)
   {"os_getenv", luv_os_getenv},
   {"os_setenv", luv_os_setenv},
@@ -301,13 +330,19 @@ static const luaL_Reg luv_functions[] = {
 #if LUV_UV_VERSION_GEQ(1, 28, 0)
   {"gettimeofday", luv_gettimeofday},
 #endif
+#if LUV_UV_VERSION_GEQ(1, 31, 0)
+  {"os_environ", luv_os_environ},
+#endif
+#if LUV_UV_VERSION_GEQ(1, 33, 0)
+  {"random", luv_random},
+#endif
+  {"sleep", luv_sleep},
 
   // thread.c
   {"new_thread", luv_new_thread},
   {"thread_equal", luv_thread_equal},
   {"thread_self", luv_thread_self},
   {"thread_join", luv_thread_join},
-  {"sleep", luv_thread_sleep},
 
   // work.c
   {"new_work", luv_new_work},
@@ -378,6 +413,9 @@ static const luaL_Reg luv_stream_methods[] = {
   {"is_readable", luv_is_readable},
   {"is_writable", luv_is_writable},
   {"set_blocking", luv_stream_set_blocking},
+#if LUV_UV_VERSION_GEQ(1, 19, 0)
+  {"get_write_queue_size", luv_stream_get_write_queue_size},
+#endif
   {NULL, NULL}
 };
 
@@ -386,7 +424,9 @@ static const luaL_Reg luv_pipe_methods[] = {
   {"bind", luv_pipe_bind},
   {"connect", luv_pipe_connect},
   {"getsockname", luv_pipe_getsockname},
+#if LUV_UV_VERSION_GEQ(1, 3, 0)
   {"getpeername", luv_pipe_getpeername},
+#endif
   {"pending_instances", luv_pipe_pending_instances},
   {"pending_count", luv_pipe_pending_count},
   {"pending_type", luv_pipe_pending_type},
@@ -420,6 +460,9 @@ static const luaL_Reg luv_tcp_methods[] = {
   {"getsockname", luv_tcp_getsockname},
   {"connect", luv_tcp_connect},
   {"write_queue_size", luv_write_queue_size},
+#if LUV_UV_VERSION_GEQ(1, 32, 0)
+  {"close_reset", luv_tcp_close_reset},
+#endif
   {NULL, NULL}
 };
 
@@ -467,6 +510,25 @@ static const luaL_Reg luv_signal_methods[] = {
   {NULL, NULL}
 };
 
+#if LUV_UV_VERSION_GEQ(1, 28, 0)
+static const luaL_Reg luv_dir_methods[] = {
+  {"readdir", luv_fs_readdir},
+  {"closedir", luv_fs_closedir},
+  {NULL, NULL}
+};
+
+static void luv_dir_init(lua_State* L) {
+  luaL_newmetatable(L, "uv_dir");
+  lua_pushcfunction(L, luv_fs_dir_tostring);
+  lua_setfield(L, -2, "__tostring");
+  lua_pushcfunction(L, luv_fs_dir_gc);
+  lua_setfield(L, -2, "__gc");
+  luaL_newlib(L, luv_dir_methods);
+  lua_setfield(L, -2, "__index");
+  lua_pop(L, 1);
+}
+#endif
+
 static void luv_handle_init(lua_State* L) {
 
   lua_newtable(L);
@@ -543,11 +605,12 @@ LUALIB_API int luv_cfpcall(lua_State* L, int nargs, int nresult, int flags) {
     ret = -ret;
     break;
   case LUA_ERRRUN:
-  case LUA_ERRSYNTAX:
   case LUA_ERRERR:
   default:
     if ((flags & LUVF_CALLBACK_NOERRMSG) == 0)
       fprintf(stderr, "Uncaught Error: %s\n", lua_tostring(L, -1));
+    if ((flags & LUVF_CALLBACK_NOEXIT) == 0)
+      exit(-1);
     lua_pop(L, 1);
     ret = -ret;
     break;
@@ -648,11 +711,13 @@ LUALIB_API int luaopen_luv (lua_State* L) {
 
     lua_pushstring(L, "_loop");
     loop = (uv_loop_t*)lua_newuserdata(L, sizeof(*loop));
-    lua_rawset(L, -3);  // ref to loop, avoid __gc early
-
-    // setup the metatable for __gc
+    // setup the userdata's metatable for __gc
     luaL_getmetatable(L, "uv_loop.meta");
     lua_setmetatable(L, -2);
+    // create a ref to loop, avoid __gc early
+    // this puts the loop userdata into the _loop key
+    // in the returned luv table
+    lua_rawset(L, -3);
 
     ctx->loop = loop;
     ctx->L = L;
@@ -669,6 +734,9 @@ LUALIB_API int luaopen_luv (lua_State* L) {
 
   luv_req_init(L);
   luv_handle_init(L);
+#if LUV_UV_VERSION_GEQ(1, 28, 0)
+  luv_dir_init(L);
+#endif
   luv_thread_init(L);
   luv_work_init(L);
 
