@@ -14,7 +14,7 @@
  *  limitations under the License.
  *
  */
-#include "luv.h"
+#include "private.h"
 
 static uv_pipe_t* luv_check_pipe(lua_State* L, int index) {
   uv_pipe_t* handle = (uv_pipe_t*)luv_checkudata(L, index, "uv_pipe");
@@ -26,8 +26,7 @@ static int luv_new_pipe(lua_State* L) {
   uv_pipe_t* handle;
   int ipc, ret;
   luv_ctx_t* ctx = luv_context(L);
-  luaL_checktype(L, 1, LUA_TBOOLEAN);
-  ipc = lua_toboolean(L, 1);
+  ipc = luv_optboolean(L, 1, 0);
   handle = (uv_pipe_t*)luv_newuserdata(L, sizeof(*handle));
   ret = uv_pipe_init(ctx->loop, handle, ipc);
   if (ret < 0) {
@@ -42,18 +41,14 @@ static int luv_pipe_open(lua_State* L) {
   uv_pipe_t* handle = luv_check_pipe(L, 1);
   uv_file file = luaL_checkinteger(L, 2);
   int ret = uv_pipe_open(handle, file);
-  if (ret < 0) return luv_error(L, ret);
-  lua_pushinteger(L, ret);
-  return 1;
+  return luv_result(L, ret);
 }
 
 static int luv_pipe_bind(lua_State* L) {
   uv_pipe_t* handle = luv_check_pipe(L, 1);
   const char* name = luaL_checkstring(L, 2);
   int ret = uv_pipe_bind(handle, name);
-  if (ret < 0) return luv_error(L, ret);
-  lua_pushinteger(L, ret);
-  return 1;
+  return luv_result(L, ret);
 }
 
 static int luv_pipe_connect(lua_State* L) {
@@ -77,6 +72,7 @@ static int luv_pipe_getsockname(lua_State* L) {
   return 1;
 }
 
+#if LUV_UV_VERSION_GEQ(1, 3, 0)
 static int luv_pipe_getpeername(lua_State* L) {
   uv_pipe_t* handle = luv_check_pipe(L, 1);
   size_t len = 2*PATH_MAX;
@@ -86,6 +82,7 @@ static int luv_pipe_getpeername(lua_State* L) {
   lua_pushlstring(L, buf, len);
   return 1;
 }
+#endif
 
 static int luv_pipe_pending_instances(lua_State* L) {
   uv_pipe_t* handle = luv_check_pipe(L, 1);
