@@ -14,7 +14,7 @@
  *  limitations under the License.
  *
  */
-#include "luv.h"
+#include "private.h"
 
 static uv_poll_t* luv_check_poll(lua_State* L, int index) {
   uv_poll_t* handle = (uv_poll_t*)luv_checkudata(L, index, "uv_poll");
@@ -48,10 +48,15 @@ static int luv_new_socket_poll(lua_State* L) {
   return 1;
 }
 
-// These are the same order as uv_run_mode which also starts at 0
 static const char *const luv_pollevents[] = {
-  "r", "w", "rw", "d", "rd", "wd", "rwd",
-  "p", "rp", "wp", "rwp", "dp", "rdp", "wdp", "rwdp", NULL
+  "r", "w", "rw",
+#if LUV_UV_VERSION_GEQ(1, 9, 0)
+  "d", "rd", "wd", "rwd",
+#endif
+#if LUV_UV_VERSION_GEQ(1, 14, 0)
+  "p", "rp", "wp", "rwp", "dp", "rdp", "wdp", "rwdp",
+#endif
+  NULL
 };
 
 static void luv_poll_cb(uv_poll_t* handle, int status, int events) {
@@ -71,10 +76,13 @@ static void luv_poll_cb(uv_poll_t* handle, int status, int events) {
     case UV_READABLE: evtstr = "r"; break;
     case UV_WRITABLE: evtstr = "w"; break;
     case UV_READABLE|UV_WRITABLE: evtstr = "rw"; break;
+#if LUV_UV_VERSION_GEQ(1, 9, 0)
     case UV_DISCONNECT: evtstr = "d"; break;
     case UV_READABLE|UV_DISCONNECT: evtstr = "rd"; break;
     case UV_WRITABLE|UV_DISCONNECT: evtstr = "wd"; break;
     case UV_READABLE|UV_WRITABLE|UV_DISCONNECT: evtstr = "rwd"; break;
+#endif
+#if LUV_UV_VERSION_GEQ(1, 14, 0)
     case UV_PRIORITIZED: evtstr = "p"; break;
     case UV_READABLE|UV_PRIORITIZED: evtstr = "rp"; break;
     case UV_WRITABLE|UV_PRIORITIZED: evtstr = "wp"; break;
@@ -83,6 +91,7 @@ static void luv_poll_cb(uv_poll_t* handle, int status, int events) {
     case UV_READABLE|UV_DISCONNECT|UV_PRIORITIZED: evtstr = "rdp"; break;
     case UV_WRITABLE|UV_DISCONNECT|UV_PRIORITIZED: evtstr = "wdp"; break;
     case UV_READABLE|UV_WRITABLE|UV_DISCONNECT|UV_PRIORITIZED: evtstr = "rwdp"; break;
+#endif
     default: evtstr = ""; break;
   }
   lua_pushstring(L, evtstr);
@@ -97,10 +106,13 @@ static int luv_poll_start(lua_State* L) {
     case 0: events = UV_READABLE; break;
     case 1: events = UV_WRITABLE; break;
     case 2: events = UV_READABLE | UV_WRITABLE; break;
+#if LUV_UV_VERSION_GEQ(1, 9, 0)
     case 3: events = UV_DISCONNECT; break;
     case 4: events = UV_READABLE|UV_DISCONNECT; break;
     case 5: events = UV_WRITABLE|UV_DISCONNECT; break;
     case 6: events = UV_READABLE|UV_WRITABLE|UV_DISCONNECT; break;
+#endif
+#if LUV_UV_VERSION_GEQ(1, 14, 0)
     case 7: events = UV_PRIORITIZED; break;
     case 8: events = UV_READABLE|UV_PRIORITIZED; break;
     case 9: events = UV_WRITABLE|UV_PRIORITIZED; break;
@@ -109,19 +121,16 @@ static int luv_poll_start(lua_State* L) {
     case 12: events = UV_READABLE|UV_DISCONNECT|UV_PRIORITIZED; break;
     case 13: events = UV_WRITABLE|UV_DISCONNECT|UV_PRIORITIZED; break;
     case 14: events = UV_READABLE|UV_WRITABLE|UV_DISCONNECT|UV_PRIORITIZED; break;
+#endif
     default: events = 0; /* unreachable */
   }
   luv_check_callback(L, (luv_handle_t*)handle->data, LUV_POLL, 3);
   ret = uv_poll_start(handle, events, luv_poll_cb);
-  if (ret < 0) return luv_error(L, ret);
-  lua_pushinteger(L, ret);
-  return 1;
+  return luv_result(L, ret);
 }
 
 static int luv_poll_stop(lua_State* L) {
   uv_poll_t* handle = luv_check_poll(L, 1);
   int ret = uv_poll_stop(handle);
-  if (ret < 0) return luv_error(L, ret);
-  lua_pushinteger(L, ret);
-  return 1;
+  return luv_result(L, ret);
 }
