@@ -14,7 +14,7 @@
  *  limitations under the License.
  *
  */
-#include "luv.h"
+#include "private.h"
 #include <math.h>
 
 static int luv_disable_stdio_inheritance(lua_State* L) {
@@ -253,6 +253,10 @@ static int luv_spawn(lua_State* L) {
   lua_pop(L, 1);
 #endif
 
+  // this will fill the 3rd argument with nil if it doesn't exist so that
+  // the uv_process_t userdata doesn't get treated as the 3rd argument
+  lua_settop(L, 3);
+
   handle = (uv_process_t*)luv_newuserdata(L, sizeof(*handle));
   handle->type = UV_PROCESS;
   handle->data = luv_setup_handle(L, ctx);
@@ -289,16 +293,12 @@ static int luv_process_kill(lua_State* L) {
   uv_process_t* handle = luv_check_process(L, 1);
   int signum = luv_parse_signal(L, 2);
   int ret = uv_process_kill(handle, signum);
-  if (ret < 0) return luv_error(L, ret);
-  lua_pushinteger(L, ret);
-  return 1;
+  return luv_result(L, ret);
 }
 
 static int luv_kill(lua_State* L) {
   int pid = luaL_checkinteger(L, 1);
   int signum = luv_parse_signal(L, 2);
   int ret = uv_kill(pid, signum);
-  if (ret < 0) return luv_error(L, ret);
-  lua_pushinteger(L, ret);
-  return 1;
+  return luv_result(L, ret);
 }
