@@ -14,7 +14,7 @@
  *  limitations under the License.
  *
  */
-#include "luv.h"
+#include "private.h"
 
 static uv_req_t* luv_check_req(lua_State* L, int index) {
   uv_req_t* req = (uv_req_t*)luaL_checkudata(L, index, "uv_req");
@@ -44,8 +44,6 @@ static void luv_req_init(lua_State* L) {
 static int luv_cancel(lua_State* L) {
   uv_req_t* req = (uv_req_t*)luv_check_req(L, 1);
   int ret = uv_cancel(req);
-  if (ret < 0) return luv_error(L, ret);
   // Cleanup occurs when callbacks are ran with UV_ECANCELED status.
-  lua_pushinteger(L, ret);
-  return 1;
+  return luv_result(L, ret);
 }
