@@ -14,7 +14,7 @@
  *  limitations under the License.
  *
  */
-#include "luv.h"
+#include "private.h"
 
 static uv_signal_t* luv_check_signal(lua_State* L, int index) {
   uv_signal_t* handle = (uv_signal_t*)luv_checkudata(L, index, "uv_signal");
@@ -59,40 +59,34 @@ static int luv_signal_start(lua_State* L) {
     luv_check_callback(L, (luv_handle_t*)handle->data, LUV_SIGNAL, 3);
   }
   ret = uv_signal_start(handle, luv_signal_cb, signum);
-  if (ret < 0) return luv_error(L, ret);
-  lua_pushinteger(L, ret);
-  return 1;
+  return luv_result(L, ret);
 }
 
 #if LUV_UV_VERSION_GEQ(1, 12, 0)
 static int luv_signal_start_oneshot(lua_State* L) {
-	uv_signal_t* handle = luv_check_signal(L, 1);
-	int signum, ret;
-	if (lua_isnumber(L, 2)) {
-		signum = lua_tointeger(L, 2);
-	}
-	else if (lua_isstring(L, 2)) {
-		signum = luv_sig_string_to_num(luaL_checkstring(L, 2));
-		luaL_argcheck(L, signum, 2, "Invalid Signal name");
-	}
-	else {
-		return luaL_argerror(L, 2, "Missing Signal name");
-	}
+  uv_signal_t* handle = luv_check_signal(L, 1);
+  int signum, ret;
+  if (lua_isnumber(L, 2)) {
+    signum = lua_tointeger(L, 2);
+  }
+  else if (lua_isstring(L, 2)) {
+    signum = luv_sig_string_to_num(luaL_checkstring(L, 2));
+    luaL_argcheck(L, signum, 2, "Invalid Signal name");
+  }
+  else {
+    return luaL_argerror(L, 2, "Missing Signal name");
+  }
 
-	if (!lua_isnoneornil(L, 3)) {
-		luv_check_callback(L, (luv_handle_t*)handle->data, LUV_SIGNAL, 3);
-	}
-	ret = uv_signal_start_oneshot(handle, luv_signal_cb, signum);
-	if (ret < 0) return luv_error(L, ret);
-	lua_pushinteger(L, ret);
-	return 1;
+  if (!lua_isnoneornil(L, 3)) {
+    luv_check_callback(L, (luv_handle_t*)handle->data, LUV_SIGNAL, 3);
+  }
+  ret = uv_signal_start_oneshot(handle, luv_signal_cb, signum);
+  return luv_result(L, ret);
 }
 #endif
 
 static int luv_signal_stop(lua_State* L) {
   uv_signal_t* handle = luv_check_signal(L, 1);
   int ret = uv_signal_stop(handle);
-  if (ret < 0) return luv_error(L, ret);
-  lua_pushinteger(L, ret);
-  return 1;
+  return luv_result(L, ret);
 }
