@@ -14,13 +14,7 @@
  *  limitations under the License.
  *
  */
-#include "luv.h"
-
-static void luv_check_buf(lua_State *L, int idx, uv_buf_t *pbuf) {
-    size_t len;
-    pbuf->base = (char*)luaL_checklstring(L, idx, &len);
-    pbuf->len = len;
-}
+#include "private.h"
 
 static uv_stream_t* luv_check_stream(lua_State* L, int index) {
   int isStream;
@@ -77,18 +71,14 @@ static int luv_listen(lua_State* L) {
   int ret;
   luv_check_callback(L, (luv_handle_t*)handle->data, LUV_CONNECTION, 3);
   ret = uv_listen(handle, backlog, luv_connection_cb);
-  if (ret < 0) return luv_error(L, ret);
-  lua_pushinteger(L, ret);
-  return 1;
+  return luv_result(L, ret);
 }
 
 static int luv_accept(lua_State* L) {
   uv_stream_t* server = luv_check_stream(L, 1);
   uv_stream_t* client = luv_check_stream(L, 2);
   int ret = uv_accept(server, client);
-  if (ret < 0) return luv_error(L, ret);
-  lua_pushinteger(L, ret);
-  return 1;
+  return luv_result(L, ret);
 }
 
 static void luv_alloc_cb(uv_handle_t* handle, size_t suggested_size, uv_buf_t* buf) {
@@ -128,17 +118,13 @@ static int luv_read_start(lua_State* L) {
   int ret;
   luv_check_callback(L, (luv_handle_t*)handle->data, LUV_READ, 2);
   ret = uv_read_start(handle, luv_alloc_cb, luv_read_cb);
-  if (ret < 0) return luv_error(L, ret);
-  lua_pushinteger(L, ret);
-  return 1;
+  return luv_result(L, ret);
 }
 
 static int luv_read_stop(lua_State* L) {
   uv_stream_t* handle = luv_check_stream(L, 1);
   int ret = uv_read_stop(handle);
-  if (ret < 0) return luv_error(L, ret);
-  lua_pushinteger(L, ret);
-  return 1;
+  return luv_result(L, ret);
 }
 
 static void luv_write_cb(uv_write_t* req, int status) {
@@ -150,19 +136,6 @@ static void luv_write_cb(uv_write_t* req, int status) {
   req->data = NULL;
 }
 
-static uv_buf_t* luv_prep_bufs(lua_State* L, int index, size_t *count) {
-  uv_buf_t *bufs;
-  size_t i;
-  *count = lua_rawlen(L, index);
-  bufs = (uv_buf_t*)malloc(sizeof(uv_buf_t) * *count);
-  for (i = 0; i < *count; ++i) {
-    lua_rawgeti(L, index, i + 1);
-    luv_check_buf(L, -1, &bufs[i]);
-    lua_pop(L, 1);
-  }
-  return bufs;
-}
-
 static int luv_write(lua_State* L) {
   luv_ctx_t* ctx = luv_context(L);
   uv_stream_t* handle = luv_check_stream(L, 1);
@@ -171,27 +144,15 @@ static int luv_write(lua_State* L) {
   ref = luv_check_continuation(L, 3);
   req = (uv_write_t *)lua_newuserdata(L, sizeof(*req));
   req->data = (luv_req_t*)luv_setup_req(L, ctx, ref);
-  if (lua_istable(L, 2)) {
-    size_t count;
-    uv_buf_t *bufs = luv_prep_bufs(L, 2, &count);
-    ret = uv_write(req, handle, bufs, count, luv_write_cb);
-    free(bufs);
-  }
-  else if (lua_isstring(L, 2)) {
-    uv_buf_t buf;
-    luv_check_buf(L, 2, &buf);
-    ret = uv_write(req, handle, &buf, 1, luv_write_cb);
-  }
-  else {
-    return luaL_argerror(L, 2, "data must be string or table of strings");
-  }
+  size_t count;
+  uv_buf_t* bufs = luv_check_bufs(L, 2, &count, (luv_req_t*)req->data);
+  ret = uv_write(req, handle, bufs, count, luv_write_cb);
+  free(bufs);
   if (ret < 0) {
     luv_cleanup_req(L, (luv_req_t*)req->data);
     lua_pop(L, 1);
     return luv_error(L, ret);
   }
-  lua_pushvalue(L, 2);
-  ((luv_req_t*)req->data)->data_ref = luaL_ref(L, LUA_REGISTRYINDEX);
   return 1;
 }
 
@@ -205,49 +166,27 @@ static int luv_write2(lua_State* L) {
   ref = luv_check_continuation(L, 4);
   req = (uv_write_t *)lua_newuserdata(L, sizeof(*req));
   req->data = luv_setup_req(L, ctx, ref);
-  if (lua_istable(L, 2)) {
-    size_t count;
-    uv_buf_t *bufs = luv_prep_bufs(L, 2, &count);
-    ret = uv_write2(req, handle, bufs, count, send_handle, luv_write_cb);
-    free(bufs);
-  }
-  else if (lua_isstring(L, 2)) {
-    uv_buf_t buf;
-    luv_check_buf(L, 2, &buf);
-    ret = uv_write2(req, handle, &buf, 1, send_handle, luv_write_cb);
-  }
-  else {
-    return luaL_argerror(L, 2, "data must be string or table of strings");
-  }
+  size_t count;
+  uv_buf_t* bufs = luv_check_bufs(L, 2, &count, (luv_req_t*)req->data);
+  ret = uv_write2(req, handle, bufs, count, send_handle, luv_write_cb);
+  free(bufs);
   if (ret < 0) {
     luv_cleanup_req(L, (luv_req_t*)req->data);
     lua_pop(L, 1);
     return luv_error(L, ret);
   }
-  lua_pushvalue(L, 2);
-  ((luv_req_t*)req->data)->data_ref = luaL_ref(L, LUA_REGISTRYINDEX);
   return 1;
 }
 
 static int luv_try_write(lua_State* L) {
   uv_stream_t* handle = luv_check_stream(L, 1);
-  int ret;
-  if (lua_istable(L, 2)) {
-    size_t count;
-    uv_buf_t *bufs = luv_prep_bufs(L, 2, &count);
-    ret = uv_try_write(handle, bufs, count);
-    free(bufs);
-  }
-  else if (lua_isstring(L, 2)) {
-    uv_buf_t buf;
-    luv_check_buf(L, 2, &buf);
-    ret = uv_try_write(handle, &buf, 1);
-  }
-  else {
-    return luaL_argerror(L, 2, "data must be string or table of strings");
-  }
-  if (ret < 0) return luv_error(L, ret);
-  lua_pushinteger(L, ret);
+  int err_or_num_bytes;
+  size_t count;
+  uv_buf_t* bufs = luv_check_bufs_noref(L, 2, &count);
+  err_or_num_bytes = uv_try_write(handle, bufs, count);
+  free(bufs);
+  if (err_or_num_bytes < 0) return luv_error(L, err_or_num_bytes);
+  lua_pushinteger(L, err_or_num_bytes);
   return 1;
 }
 
@@ -269,7 +208,13 @@ static int luv_stream_set_blocking(lua_State* L) {
   luaL_checktype(L, 2, LUA_TBOOLEAN);
   blocking = lua_toboolean(L, 2);
   ret = uv_stream_set_blocking(handle, blocking);
-  if (ret < 0) return luv_error(L, ret);
-  lua_pushinteger(L, ret);
+  return luv_result(L, ret);
+}
+
+#if LUV_UV_VERSION_GEQ(1, 19, 0)
+static int luv_stream_get_write_queue_size(lua_State* L) {
+  uv_stream_t* handle = luv_check_stream(L, 1);
+  lua_pushinteger(L, uv_stream_get_write_queue_size(handle));
   return 1;
 }
+#endif
