@@ -14,73 +14,67 @@
 *  limitations under the License.
 *
 */
-#include "luv.h"
-#include "lthreadpool.h"
+#include "private.h"
 
 typedef struct {
   lua_State* L;       /* vm in main */
   char* code;         /* thread entry code */
   size_t len;
 
-  uv_async_t async;
-  int async_cb;       /* ref, run in main, call when async message received */
   int after_work_cb;  /* ref, run in main ,call after work cb*/
+  int pool_ref;       /* ref of lua_State cache array */
 } luv_work_ctx_t;
 
 typedef struct {
   uv_work_t work;
   luv_work_ctx_t* ctx;
 
-  luv_thread_arg_t arg;
+  luv_thread_arg_t args;
+  luv_thread_arg_t rets;
   int ref;            /* ref to luv_work_ctx_t, which create a new uv_work_t*/
 } luv_work_t;
 
-static uv_key_t L_key;
-static uv_once_t L_init = UV_ONCE_INIT;
-
-static luv_work_ctx_t* luv_check_work_ctx(lua_State* L, int index)
-{
+static luv_work_ctx_t* luv_check_work_ctx(lua_State* L, int index) {
   luv_work_ctx_t* ctx = (luv_work_ctx_t*)luaL_checkudata(L, index, "luv_work_ctx");
   return ctx;
 }
 
-static int luv_work_ctx_gc(lua_State *L)
-{
+static int luv_work_ctx_gc(lua_State *L) {
+  int i, n;
   luv_work_ctx_t* ctx = luv_check_work_ctx(L, 1);
   free(ctx->code);
   luaL_unref(L, LUA_REGISTRYINDEX, ctx->after_work_cb);
-  luaL_unref(L, LUA_REGISTRYINDEX, ctx->async_cb);
 
+  lua_rawgeti(L, LUA_REGISTRYINDEX, ctx->pool_ref);
+  n = lua_rawlen(L, -1);
+  for (i=1; i<=n; i++) {
+    lua_State *S;
+    lua_rawgeti(L, -1, i);
+    S = *(lua_State**)lua_touserdata(L, -1);
+    release_vm_cb(S);
+    lua_pop(L, 1);
+  }
+  luaL_unref(L, LUA_REGISTRYINDEX, ctx->pool_ref);
   return 0;
 }
 
-static int luv_work_ctx_tostring(lua_State* L)
-{
+static int luv_work_ctx_tostring(lua_State* L) {
   luv_work_ctx_t* ctx = luv_check_work_ctx(L, 1);
   lua_pushfstring(L, "luv_work_ctx_t: %p", ctx);
   return 1;
 }
 
-static void luv_work_cb(uv_work_t* req)
-{
-  int top;
-
+static void luv_work_cb(uv_work_t* req) {
   luv_work_t* work = (luv_work_t*)req->data;
   luv_work_ctx_t* ctx = work->ctx;
-  lua_State *L = (lua_State *)uv_key_get(&L_key);
+  lua_State *L = work->args.L;
 
-  if (L == NULL) {
-    /* vm reuse in threadpool */
-    L = acquire_vm_cb();
-    uv_key_set(&L_key, L);
-  }
-  top = lua_gettop(L);
+  int top = lua_gettop(L);
 
   /* push lua function */
   lua_pushlstring(L, ctx->code, ctx->len);
   lua_rawget(L, LUA_REGISTRYINDEX);
-  if (lua_isnil(L, -1))
-  {
+  if (lua_isnil(L, -1)) {
     lua_pop(L, 1);
 
     lua_pushlstring(L, ctx->code, ctx->len);
@@ -90,34 +84,32 @@ static void luv_work_cb(uv_work_t* req)
       lua_pop(L, 2);
 
       lua_pushnil(L);
-    } else
-    {
+    } else {
       lua_pushvalue(L, -1);
       lua_insert(L, lua_gettop(L) - 2);
       lua_rawset(L, LUA_REGISTRYINDEX);
     }
   }
 
-  if (lua_isfunction(L, -1))
-  {
-    int i = luv_thread_arg_push(L, &work->arg, 0);
-    i = luv_cfpcall(L, i, LUA_MULTRET, LUVF_CALLBACK_NOEXIT);
-    luv_thread_arg_clear(NULL, &work->arg, 0);
+  if (lua_isfunction(L, -1)) {
+    int i = luv_thread_arg_push(L, &work->args, LUVF_THREAD_SIDE_CHILD);
+    i = luv_cfpcall(L, i, LUA_MULTRET, 0);
     if ( i>=0 ) {
       //clear in main threads, luv_after_work_cb
-      i = luv_thread_arg_set(L, &work->arg, top + 1, lua_gettop(L), 0);
+      i = luv_thread_arg_set(L, &work->rets, top + 1, lua_gettop(L), LUVF_THREAD_SIDE_CHILD);
       lua_pop(L, i);  // pop all returned value
-    } else if(i==-LUA_ERRMEM) {
-      release_vm_cb(L);
-      uv_key_set(&L_key, NULL);
-      L = NULL;
+      luv_thread_arg_clear(L, &work->rets, LUVF_THREAD_SIDE_CHILD);
     }
+    luv_thread_arg_clear(L, &work->args, LUVF_THREAD_SIDE_CHILD);
   } else {
     fprintf(stderr, "Uncaught Error: %s can't be work entry\n",
-      lua_typename(L, lua_type(L,-1)));
+            lua_typename(L, lua_type(L,-1)));
     lua_pop(L, 1);
-    luv_thread_arg_clear(NULL, &work->arg, 0);
+    luv_thread_arg_clear(L, &work->args, LUVF_THREAD_SIDE_CHILD);
   }
+  work->args.L = L;
+  if (top!=lua_gettop(L))
+    luaL_error(L, "stack not balance in luv_work_cb, need %d but %d", top, lua_gettop(L));
 }
 
 static void luv_after_work_cb(uv_work_t* req, int status) {
@@ -129,30 +121,25 @@ static void luv_after_work_cb(uv_work_t* req, int status) {
   (void)status;
 
   lua_rawgeti(L, LUA_REGISTRYINDEX, ctx->after_work_cb);
-  i = luv_thread_arg_push(L, &work->arg, 0);
+  i = luv_thread_arg_push(L, &work->rets, LUVF_THREAD_SIDE_MAIN);
   luv_cfpcall(L, i, 0, 0);
 
+  //cache lua_State to reuse
+  lua_rawgeti(L, LUA_REGISTRYINDEX, ctx->pool_ref);
+  i = lua_rawlen(L, -1);
+  *(lua_State**)lua_newuserdata(L, sizeof(lua_State*)) = work->args.L;
+  lua_rawseti(L, -2, i+1);
+  lua_pop(L, 1);
+
   //ref down to ctx, up in luv_queue_work()
   luaL_unref(L, LUA_REGISTRYINDEX, work->ref);
   work->ref = LUA_NOREF;
 
-  luv_thread_arg_clear(NULL, &work->arg, 0);
+  luv_thread_arg_clear(L, &work->args, LUVF_THREAD_SIDE_MAIN);
+  luv_thread_arg_clear(L, &work->rets, LUVF_THREAD_SIDE_MAIN);
   free(work);
 }
 
-static void async_cb(uv_async_t *handle)
-{
-  luv_work_t* work = (luv_work_t*)handle->data;
-  luv_work_ctx_t* ctx = work->ctx;
-  lua_State* L = ctx->L;
-  int i;
-
-  lua_rawgeti(L, LUA_REGISTRYINDEX, ctx->async_cb);
-  i = luv_thread_arg_push(L, &work->arg, 0);
-
-  luv_cfpcall(L, i, 0, 0);
-}
-
 static int luv_new_work(lua_State* L) {
   size_t len;
   const char* buff;
@@ -172,15 +159,14 @@ static int luv_new_work(lua_State* L) {
 
   lua_pushvalue(L, 2);
   ctx->after_work_cb = luaL_ref(L, LUA_REGISTRYINDEX);
-  if (lua_gettop(L) == 4) {
-    lua_pushvalue(L, 3);
-    ctx->async_cb = luaL_ref(L, LUA_REGISTRYINDEX);
-    uv_async_init(luv_loop(L), &ctx->async, async_cb);
-  } else
-    ctx->async_cb = LUA_REFNIL;
+
   ctx->L = L;
   luaL_getmetatable(L, "luv_work_ctx");
   lua_setmetatable(L, -2);
+
+  lua_newtable(L);
+  ctx->pool_ref = luaL_ref(L, LUA_REGISTRYINDEX);
+
   return 1;
 }
 
@@ -188,9 +174,28 @@ static int luv_queue_work(lua_State* L) {
   int top = lua_gettop(L);
   luv_work_ctx_t* ctx = luv_check_work_ctx(L, 1);
   luv_work_t* work = (luv_work_t*)malloc(sizeof(*work));
-  int ret;
+  int ret, n;
 
-  luv_thread_arg_set(L, &work->arg, 2, top, 0); //clear in sub threads,luv_work_cb
+  //prepare lua_State for threadpool
+  lua_rawgeti(L, LUA_REGISTRYINDEX, ctx->pool_ref);
+  n = lua_rawlen(L, -1);
+  if (n > 0) {
+    int i;
+    lua_rawgeti(L, -1, 1);
+    work->args.L = *(lua_State **)lua_touserdata(L, -1);
+    lua_pop(L, 1);
+    for(i=1; i<n; i++) {
+      lua_rawgeti(L, -1, i+1);
+      lua_rawseti(L, -2, i);
+    }
+    lua_pushnil(L);
+    lua_rawseti(L, -2, n);
+  }
+  else
+    work->args.L = acquire_vm_cb();
+  lua_pop(L, 1);
+
+  luv_thread_arg_set(L, &work->args, 2, top, LUVF_THREAD_SIDE_MAIN); //clear in sub threads,luv_work_cb
   work->ctx = ctx;
   work->work.data = work;
   ret = uv_queue_work(luv_loop(L), &work->work, luv_work_cb, luv_after_work_cb);
@@ -212,20 +217,13 @@ static const luaL_Reg luv_work_ctx_methods[] = {
   {NULL, NULL}
 };
 
-static void luv_thread_key_init() {
-  uv_key_create(&L_key);
-}
-
 static void luv_work_init(lua_State* L) {
   luaL_newmetatable(L, "luv_work_ctx");
   lua_pushcfunction(L, luv_work_ctx_tostring);
   lua_setfield(L, -2, "__tostring");
   lua_pushcfunction(L, luv_work_ctx_gc);
   lua_setfield(L, -2, "__gc");
-  lua_newtable(L);
-  luaL_setfuncs(L, luv_work_ctx_methods, 0);
+  luaL_newlib(L, luv_work_ctx_methods);
   lua_setfield(L, -2, "__index");
   lua_pop(L, 1);
-
-  uv_once(&L_init, luv_thread_key_init);
 }
